--[[
	@brief: Render function unit.
	@param：RenderUnit.lua
	@date：Created by Dongyang Cai on 26/9/2014
]]

RenderUnit = RenderUnit or BaseClass()

function RenderUnit:__init(name)
	self.core_game_scene = nil
	self.name = name

	--初始逻辑块大小为32，以后加载场景了要设置成场景的逻辑块大小
	self.logic_tile_size = cc.size(60,30)
	self.world_size = cc.size(0, 0)

	self.rtt_size = cc.Director:getInstance():getVisibleSize()
	self.rtt_size_width = self.rtt_size.width
	self.rtt_size_height = self.rtt_size.height
	self.rtt_width_half = self.rtt_size_width*0.5
	self.rtt_height_half = self.rtt_size_height*0.5

	--使用渲染到纹理时才创建
	self.rtt_helper = nil

	--opengl 视口大小
	self.design_resolution = self.rtt_size
	--ui 美术设计出图期望分辨率
	self.expect_resolution = cc.size(1024,640)

	--全屏设计的界面 视口到设计的放缩比
	--考虑到像素对齐及字体的清新度不作统一的放缩
	--resolution_radio 小于1.0视口会容不下UI，要作调整；如根节点setScale(resolution_radio)
	--对于高度640设计的半全屏窗口(宽度较小窗口 如聊天，背包)如果 resolution_radio_y 小于1.0 根节点setScale(resolution_radio_y)
	--大于1.0的就UI不会全屏，可根据需求选择处理
	self.resolution_radio_x = self.design_resolution.width/self.expect_resolution.width
	self.resolution_radio_y = self.design_resolution.height/self.expect_resolution.height
	self.resolution_radio = math.min(self.resolution_radio_x, self.resolution_radio_y)
end

function RenderUnit:__delete()
	self.core_game_scene = nil

	if self.rtt_helper then
		self.rtt_helper:stopRender()
		self.rtt_helper = nil
	end
end

--[[
	@brief: 获取渲染器名称
]]
function RenderUnit:GetName()
	return self.name
end

--[[
	@brief: 初始化主渲染器
]]
function RenderUnit:InitAsMainStage()
	self.core_game_scene = cc.GameScene:create()
	cc.Director:getInstance():runWithScene(self.core_game_scene)
	self.camera = self.core_game_scene:getCamera()

	-- self.ui_scale = self.core_game_scene:getRenderGroup(cc.RenderQueue.GRQ_UI):getScale()
    self.ui_scale = 1.0
	self.ui_scale_inv = 1.0/self.ui_scale
	self.winSize = cc.size(self.rtt_size.width * self.ui_scale_inv, self.rtt_size.height * self.ui_scale_inv)
end

function RenderUnit:SetWorldSize(sz)
	self.world_size = sz
	self.world_size_width = sz.width
	self.world_size_height = sz.height
end

function RenderUnit:SetLogicTileSize(sz)
	self.logic_tile_sz = sz
end

function RenderUnit:GetCoreScene()
	return self.core_game_scene
end

function RenderUnit:GetMainCamera()
	return self.core_game_scene:getCamera()
end

function RenderUnit:SetCameraPos(x, y)
	y = self.world_size_height - y
	self.camera:setEyeXY(x, y)
end

function RenderUnit:GetCameraPos()
	local eye = self.camera:getEye()
	eye.y = self.world_size_height - eye.y
	return eye.x, eye.y
end

function RenderUnit:GetWidth()
	return self.rtt_size.width
end

function RenderUnit:GetHeight()
	return self.rtt_size.height
end

function RenderUnit:GetVisibleSize( )
	return self.rtt_size_width, self.rtt_size_height
end

function RenderUnit:getWinSize( )
	return self.winSize
end

function RenderUnit:FlipWorldPosPoint(pos)
	pos.y = self.world_size_height - pos.y
	return pos
end

function RenderUnit:FlipWorldPosXY(x, y)
	y = self.world_size_height - y
	return x, y
end

function RenderUnit:LogicToWorld(logic_pos)
	local wpos = cc.vec2(logic_pos.x, logic_pos.y)
	wpos.x = wpos.x * self.logic_tile_width
	wpos.y = wpos.y * self.logic_tile_height
	--wpos = wpos + cc.vec2(self.logic_tile_size.width/2, self.logic_tile_size.height/2)
	--self:FlipWorldPosPoint(wpos)
	wpos.y = self.world_size_height - wpos.y
	return wpos
end

function RenderUnit:KmLogicToWorld(logic_pos)
	local wpos = cc.vec2(logic_pos.x, logic_pos.y)
	wpos.x = wpos.x * self.logic_tile_width
	wpos.y = wpos.y * self.logic_tile_height
	--wpos = wpos + cc.vec2(self.logic_tile_size.width/2, self.logic_tile_size.height/2)
	--self:FlipWorldPosPoint(wpos)
	wpos.y = self.world_size_height - wpos.y
	return wpos
end

function RenderUnit:WorldToLogic(wpos)
	local lpos = cc.vec2(wpos.x, wpos.y)
	lpos.y = self.world_size_height - lpos.y
	lpos.x = lpos.x / self.logic_tile_width
	lpos.y = lpos.y / self.logic_tile_height
	return lpos
end

function RenderUnit:LogicToPixel(logic_pos)
	local pix_pos = cc.vec2(0.5, 0.5) + logic_pos
	return pix_pos
end

function RenderUnit:PixelToLogic(pix_pos)
	local logic_pos = cc.vec2(math.floor(pix_pos.x), math.floor(pix_pos.y))
	return logic_pos
end

function RenderUnit:ViewToWorld(view_pt)
    -- return self.core_game_scene:screenToWorld(view_pt.x, view_pt.y)
    local pos = self.core_game_scene:screenToWorld(view_pt.x, view_pt.y)
    return cc.vec2(pos.x , pos.y)
end

function RenderUnit:worldToUI( world_pos )
	return self.core_game_scene:worldToUI(world_pos.x, world_pos.y)
end

function RenderUnit:UIToWorld(ui_pos)
	return self.core_game_scene:uiToWorld(ui_pos.x, ui_pos.y)
end

function RenderUnit:TouchToScreen(touch_pos)
	return cc.vec2(touch_pos.x - self.rtt_size.width / 2 , self.rtt_size.height / 2 - touch_pos.y)
end


function RenderUnit:TouchToUI(touch_pos)
	return cc.vec2(touch_pos.x - self.rtt_size.width / 2 , self.rtt_size.height / 2 - touch_pos.y)
end

function RenderUnit:getResolutionScale()
	return self.resolution_radio
end