--利用ID分离对EventSystem的直接引用
_inner_event_system_list = _inner_event_system_list or {}
_inner_event_system_id_count = _inner_event_system_id_count or 0

--用来标识是否是事件的handle
_inner_event_connection_obj = _inner_event_connection_obj or {}

EventSystem = EventSystem or BaseClass()

--事件系统(非单健)
function EventSystem:__init()
	_inner_event_system_id_count = _inner_event_system_id_count + 1
	self.system_id = _inner_event_system_id_count
	_inner_event_system_list[self.system_id] = self

	--事件列表
	self.event_list = {}

	--需要激发的事件(延后调用方式)
	self.need_fire_events = List.New()
end

function EventSystem:__delete()
	_inner_event_system_list[self.system_id] = nil
	-- self.event_list = nil
	self.event_list = {}
end

--调用已经处于派发队列中的Event
function EventSystem:Update()
	--依次执行所有需要触发的事件
	while not List.Empty(self.need_fire_events) do
		local fire_info = List.PopFront(self.need_fire_events)
		self:Fire(fire_info.event_id, unpack(fire_info.arg_list))
	end
end

function EventSystem:Bind(event_id, event_func)
	if event_id == nil then
		error("Try to bind to a nil event_id")
		return
	end

	self.event_list[event_id] = self.event_list[event_id] or {}
	local event = self.event_list[event_id]
	local bind_id = #event + 1
	event[bind_id] = event_func

	local handle = {}
	setmetatable(handle, _inner_event_connection_obj)
	handle.dispatcher = self
	handle.event_id = event_id
	handle.bind_id = bind_id
	return handle
end

function EventSystem:UnBind(handle)
	if handle == nil or handle.event_id == nil then
		-- CCINFO(debug.traceback())
		error("Try to unbind a nil event_id")
		return
	end
	local event = self.event_list[handle.event_id]
	if event ~= nil then
		--仅当handle符合类型时才作对应操作
		if getmetatable(handle) == _inner_event_connection_obj then
			event[handle.bind_id] = nil
		end
	end
end

--立即触发
function EventSystem:Fire(event_id, ...)
	if event_id == nil then
		error("Try to call EventSystem:Fire() with a nil event_id")
		return
	end
	if not self.event_list then
		print("[Error]the vo event_list is nil!")
		print(debug.traceback())
		return
	end
	local event = self.event_list[event_id] 
	if event ~= nil then
		for _, func in pairs(event) do
			func(...)
		end
	else
		-- print(debug.traceback())
	end
end

--下一帧触发
function EventSystem:FireNextFrame(event_id, ...)
	if event_id == nil then
		error("Try to call EventSystem:FireNextFrame() with a nil event_id")
		return
	end

	local fire_info = {}
	fire_info.event_id = event_id
	fire_info.arg_list = {...}
	List.PushBack(self.need_fire_events, fire_info)
end
