--[[
	@brief: 根据外部设定的优先级在每一帧中依次执行所有托管的RunObj.
	@param：main.lua
	@date：Created by Dongyang Cai on 26/9/2014
]]

Runner = Runner or BaseClass()

function Runner:__init(  )
	Runner._instance = self
	--用于标记某个模块是否已经注册,避免重复性的注册
	self.all_run_obj_list = {}
	self.id_count = 0

	--出现每帧间隔时间大于0.1秒的次数 
	self.num50  = 0
	--出现每帧间隔时间大于0.5秒的次数 
	self.num10 = 0

	--支持1 ~ 4级优先级指定, 1为最先执行, 4为最后执行
	self.priority_run_obj_list = {}
	for i=1,4 do
		table.insert(self.priority_run_obj_list, {})
	end
end

function Runner:getInstance()
	if Runner._instance == nil then
		Runner.New()
	end
	return Runner._instance
end

--[[
	@brief: 主Update中调用该方法,触发托管对象的Update
]]
function Runner:Update( now_time, elapse_time )
	for i=1,4 do
		local priority_tbl = self.priority_run_obj_list[i]
		for _, v in pairs(priority_tbl) do
			v:Update(now_time, elapse_time)
		end
	end
end

--[[
	@brief: 向Runner添加一个RunObj, RunObj必须存在Update方法
	@param: run_obj 要添加的run_obj对象 any 必须实现Update方法
			priority_level Update优先级 1-4,数字越小越先执行
	@return: 是否添加run_obj成功 bool run_obj如果已经存在于Runner中, 则不能再添加
]]
function Runner:AddRunObj( run_obj , priority_level )
	local obj = self.all_run_obj_list[run_obj]
	if obj ~= nil then
		--已经存在该对象, 不重复添加
		return false
	else
		if run_obj["Update"] == nil then
			error("Runner:AddRunObj try to add a obj not have Update method!")
		end

		--对象不存在,正常添加
		self.id_count = self.id_count + 1
		priority_level = priority_level or 4
		self.all_run_obj_list[run_obj] = {priority_level, self.id_count}
		self.priority_run_obj_list[priority_level][self.id_count] = run_obj
	end
end

--[[
	@brief: 从Runner中删除一个run_obj
	@param: run_obj
]]
function Runner:RemoveRunObj( run_obj )
	local key_info = self.all_run_obj_list[run_obj]
	if key_info ~= nil then
		self.all_run_obj_list[run_obj] = nil
		self.priority_run_obj_list[key_info[1]][key_info[2]] = nil
	end
end

--[[
	@brief: 获取添加的帧频总数
	@param: 添加的帧频总数
]]
function Runner:GetRunnerNum(  )
	local runner_number = 0
	for i=1,4 do
		local priority_tbl = self.priority_run_obj_list[i]
		for _, v in pairs(priority_tbl) do
			runner_number = runner_number + 1
		end
	end
	return runner_number
end