--[[
    @brief: Game udpate view.
    @param：update/GameUpdate.lua
    @date：Created by Dongyang Cai on 4/11/2014
]]

require("update.SAlert")
require("update.LevelUpdateView")

GameUpdate = GameUpdate or BaseClass()

GameUpdate.load_level_num = nil
GameUpdate.level_pack_next = nil

function GameUpdate:__init()
    --网络连接失败重连次数
    self.fail_connect_number = 0
    --域名类型
    self.domain_type = 1
    --版本号
    self.version = 1
    --更新进度
    self.percent = 0
    --域名设置
    self.domain = Config.Update.DOMAIN_LIST[self.domain_type]
    Config.PHP.resetPHPDomain(self.domain)
    --版本信息info
    self.version_info = {}
    --获取版本信息后的回调
    self.version_callback = nil

    self:initUpdate()
    self:configUI()
end

function GameUpdate:__delete()
    Runner:getInstance():RemoveRunObj(self)

    if self.update_light then
        self.update_light:removeFromParent()
        self.update_light = nil
    end
    if self.root_wnd then
        self.root_wnd:removeFromParent()
        self.root_wnd = nil
    end
end

--[[
    @brief: 初始化更新配置
]]
function GameUpdate:initUpdate()
    --内部测试
    local diyVersion = cc.UserDefault:getInstance():getIntegerForKey("diyVersion")
    if diyVersion and diyVersion > 0 then
        self.version = diyVersion
        cc.UserDefault:getInstance():setIntegerForKey("diyVersion", 0)
        cc.UserDefault:getInstance():flush()
        print("===>> self.version", diyVersion, self.version)
    end

    --初始化获取版本信息回调
    local function version_callback(result, diff_pack_cnt, local_version, remote_version, local_app_version, remote_app_version, app_size)
        print("+++++++++++++++version_callback")
        print(result, diff_pack_cnt,  local_version, remote_version, local_app_version, remote_app_version, app_size)
        self.version_info.result = result
        self.version_info.diff_pack_count = diff_pack_cnt
        self.version_info.local_version = local_version
        self.version_info.remote_version = remote_version
        self.version_info.local_app_version = local_app_version
        self.version_info.remote_app_version = remote_app_version
        self.version_info.app_size = app_size
        self.version_info.diff_patch_array = VersionManager:getInstance():getDiffArray()
        self.version_info.local_level, self.version_info.level_diff_array = VersionManager:getInstance():GetLevelDiffPack(1, 1)
        self.version_loading = false

        self.version_info.loaded_size = 0
        self.version_info.total_size = 0
        for i = 1, #self.version_info.diff_patch_array do
            self.version_info.total_size = self.version_info.total_size + self.version_info.diff_patch_array[i]
        end
        self.version_info.total_size = self.version_info.total_size / 1024

        print(self.version_info.local_level)
        PrintTable(self.version_info.level_diff_array)
    end
    self.version_callback = version_callback
end

function GameUpdate:configUI()
    --初始化UI
    self.root_wnd = cc.Node:create()
    self.root_wnd:setScale(HandleRenderUnit:getResolutionScale())
    HandleRenderUnit:GetCoreScene():addToRenderGroup(self.root_wnd, cc.RenderQueue.GRQ_UI)
    self.root_wnd:getParent():reorderChild(self.root_wnd, 2)
    Runner:getInstance():AddRunObj(self, 3)

    --更新和加载的背景图片
    self.loginBg = cc.SpriteUI:createWithFile("assets/icon/update/createidbg.jpg")
    self.root_wnd:addChild(self.loginBg)

    local is_bg_loaded, is_bar_loaded = false, false
    --UI加载完成后获取远程版本信息
    local check_all_loaded = function ()
        if is_bg_loaded and is_bar_loaded then
            self.isBuildedUI = true
            cc.ResService:getInstance():SetRemoteUrlPath("http://sg91.vcmedia.vn/" .. Config.PHP.ResourceID .. "/")
            local function patch_update(patch_i, patch_count, succeed, percent, speed)
                self:PatchUpdate(patch_i, patch_count, succeed, percent, speed)
            end
            VersionManager:getInstance():SetUpdateCallBack(patch_update)

            self.once_frame_action = function ()
                VersionManager:getInstance():InitDB()
            end
        end
    end

    --背景加载完成回调
    local bg_loaded_handler = function ()
        is_bg_loaded = true
        check_all_loaded()
    end
    --进度条加载完成回调
    local bar_loaded_handler = function ()
        is_bar_loaded = true
        self:setProgress(0)
        check_all_loaded()
    end

    --更新进度条背景
    self.update_bg = cc.SpriteUI:createWithResCSpriteLoaderNotify("assets/icon/update/update_bg.png", "", bg_loaded_handler)
    self.update_bg:setPosition(0, -120)
    self.root_wnd:addChild(self.update_bg)


    --进度条描述
    self.update_text = cc.Label:createWithTTF(Config.Update.CONST_DES[1], "Arial", 20)
    self.update_text:setPosition(0, -80)
    self.update_text:setColor(cc.c3b(0, 0, 0))
    self.root_wnd:addChild(self.update_text, 1)

    --进度Text
    self.percent_text = cc.Label:createWithTTF("", "Arial", 20)
    self.percent_text:setPosition(0, -120)
    self.root_wnd:addChild(self.percent_text, 1)

    --进度条progress
    self.update_bar = cc.SpriteUI:createWithResCSpriteLoaderNotify("assets/icon/update/update_bar.png", "", bar_loaded_handler)
    self.update_bar:setAnchorPoint(cc.p(0, 0.5))
    self.update_bar:setPosition(-241, -124)
    self.root_wnd:addChild(self.update_bar)

    --进度条thumb
    self.update_light_action = cc.ResSpriteAnimate:create("assets/icon/update/update_light.csprite", "default")
    self.update_light = cc.Sprite:create()
    self.update_light:setPosition(0, 20)
    self.update_bar:addChild(self.update_light, 1)
    self.update_light:runAction(self.update_light_action)

    --最新版本
    self.label1 = cc.Label:createWithTTF(Config.Update.CONST_DES[2], "Arial", 18)
    self.label1:setPosition(-50, -160)
    self.label1:setColor(cc.c3b(0, 0, 0))
    self.root_wnd:addChild(self.label1, 1)

    self.remote_version = cc.Label:createWithTTF("", "Arial", 18)
    self.remote_version:setPosition(30, -160)
    self.remote_version:setAnchorPoint(cc.p(0, 0.5))
    self.remote_version:setColor(cc.c3b(0, 0, 0))
    self.root_wnd:addChild(self.remote_version, 1)

    --当前版本
    self.label2 = cc.Label:createWithTTF(Config.Update.CONST_DES[3], "Arial", 18)
    self.label2:setPosition(-50, -180)
    self.label2:setColor(cc.c3b(0, 0, 0))
    self.root_wnd:addChild(self.label2, 1)

    self.local_version = cc.Label:createWithTTF("", "Arial", 18)
    self.local_version:setPosition(30, -180)
    self.local_version:setAnchorPoint(cc.p(0, 0.5))
    self.local_version:setColor(cc.c3b(0, 0, 0))
    self.root_wnd:addChild(self.local_version, 1)

    --获取网络类型(WIFI,GSM...)
    if self.network_type == -1 then
        print("====== > net work disconnect")
        SAlert.New(Config.Update.LOGIN_ALERT[2], function() cc.Director:getInstance():endToLua() end, Config.Update.CONST_DES[4])
    end
end

--[[
    @brief: 帧刷新函数
]]
function GameUpdate:Update(now_time, elapsed_time)
    if self.once_frame_action then
        local once_frame_action = self.once_frame_action
        self.once_frame_action()
        if self.once_frame_action == once_frame_action then
            self.once_frame_action = nil
        end
    end

    if self.frame_update then
        if self.frame_update(now_time, elapsed_time) then
            self.frame_update = nil
        end
    end

    if self.delay_call and now_time-self.delay_call.beg_time>self.delay_call.delay then
        self.delay_call.func()
        self.delay_call = nil
    end

    if self.targetValue then
        if self.percent >= self.targetValue then
            self.targetValue = nil
        else
            self.percent = self.percent + 0.1
            self.percent_text:setString(math.ceil(self.percent) .. " %")
            self.update_bar:setTextureRect(cc.rect(0, 0, 4.79*self.percent, 34))
            self.update_light:setPosition(4.79*self.percent, 20)
        end
    end

    if self.modules_index then
        if self.modules_index > self.modules_lenght then
            --模块加载完成，进入用户登陆界面
            UpdateModel:getInstance():Fire(UpdateModel.SHOW_VIEW, 3)
            self:DeleteMe()
            PackageName = "vn.shgame.sg91"
            return
        else
            require(ModulesController.module_list[self.modules_index])
            self:setProgress(math.ceil(self.modules_index/self.modules_lenght*100))
            self.modules_index = self.modules_index + 1
        end
    end
end

--[[
    @brief: 更新进度条显示
    @param: value   当前进度
]]
function GameUpdate:setProgress(value)
    if value == nil then
        return
    end
    self.targetValue = nil
    value = math.min(math.ceil(value), 100)
    self.percent_text:setString(value.." %")
    self.update_bar:setTextureRect(cc.rect(0, 0, 4.79*value, 34))
    self.update_light:setPosition(4.79*value, 20)
end

--[[
    @brief: 获取版本信息
    @param: domain  域名
]]
function GameUpdate:getVersionInfo(domain)
    local my_version_url = Config.PHP.getRemoteVersion(domain)
    VersionManager:getInstance():GetVersionInfo(self.version_callback, my_version_url)
    self.version_loading = true
end

--[[
    @brief: 重新获取版本信息
]]
function GameUpdate:retryVersionInfo()
    self.version_info = {}
    self.update_text:setString(Config.Update.LOGIN_ALERT[5])
    self:getVersionInfo(self.domain)
    self.targetValue = 98
    local total_elapsed = 0
    self.frame_update = function (now_time, elapsed_time)
        total_elapsed = total_elapsed + elapsed_time

        if self.version_info.result then
            if self.version_info.result ~= Config.Update.VERSION_DIFFTYPE.HTTP_ERROR then
                Config.PHP.resetPHPDomain(self.domain)
                self:setProgress(100)
                self:checkUpdate()
                return true
            elseif total_elapsed > 2 then
                if self.fail_connect_number >=3 and self.retryAlert == nil then
                    local function endluagame()
                        cc.Director:getInstance():endToLua()
                    end
                    local function retryconnect()
                        self.retryAlert:DeleteMe()
                        self.retryAlert = nil
                        self.percent = 0
                        self.fail_connect_number = 0
                        self:retryVersionInfo()
                    end
                    self.retryAlert = SAlert.New(Config.Update.LOGIN_ALERT[4], endluagame, Config.Update.CONST_DES[6], retryconnect, Config.Update.CONST_DES[7])
                elseif self.retryAlert == nil then
                    self.domain_type = self.domain_type == 1 and 2 or 1
                    self.domain = Config.Update.DOMAIN_LIST[self.domain_type]
                    self:getVersionInfo(self.domain)
                    self:retryVersionInfo()
                    self.fail_connect_number = self.fail_connect_number + 1
                end
            end
        end
    end
end

--[[
    @brief: 加载模块，更新加载进度
]]
function GameUpdate:requireModules()
    require("game.common.ModulesController")
    --提交错误日志
    ModulesController.submitLog()
    self.modules_index = 1
    self.modules_lenght = #ModulesController.module_list
end

--[[
    @brief: 更新完成
]]
function GameUpdate:UpdateFinished()
    if self.version_info.local_app_version and self.version_info.remote_app_version then
        if self.version_info.local_app_version < self.version_info.remote_app_version then
            print("local_app_version < remote_app_version", self.version_info.local_app_version, self.version_info.remote_app_version)
            self:loadNewApp()
            return
        end
    end

    local function UpdateEnd()
        GameUpdate.level_pack_next = GameUpdate.level_pack_next or 1000
        self.update_text:setString(Config.Update.LOGIN_ALERT[6])
        self:setProgress(0)
        self:requireModules()
    end

    if not self.maxLevel then
        self.maxLevel = cc.UserDefault:getInstance():getStringForKey("current_max_level")
        self.maxLevel = tonumber(self.maxLevel)
        self.maxLevel = self.maxLevel or 0
    end
    local dirrLen = #self.version_info.level_diff_array
    --print("----------------- MAX LEVEL", self.maxLevel, self.version_info.local_level)

    local loadLvel = 0
    local loadSize = 0
    local patch_count = 1
    while patch_count < dirrLen and loadLvel == 0 do
         local nextLevel = tonumber(self.version_info.level_diff_array[patch_count])
         loadSize = self.version_info.level_diff_array[patch_count + 1]
         if nextLevel > self.version_info.local_level then
            if self.maxLevel >= nextLevel then
                loadLvel = nextLevel
            else
                GameUpdate.level_pack_next = nextLevel
            end
            break
         end
        patch_count = patch_count + 2
    end
    if loadLvel ~= 0 then
        local level_view = LevelUpdateView.New("self_managed")
        local function closeUpdateLevel()
            GameUpdate.level_pack_next = loadLvel
            UpdateEnd()
            level_view:DeleteMe()
        end
        local function startloadLevel()
            self.patch_i = 1
            self.version_info.diff_patch_array[1] = loadSize
            if VersionManager:getInstance():LoadLevelPack(loadLvel) then
                GameUpdate.load_level_num = loadLvel
                self.version_info.local_level = loadLvel
            else
                closeUpdateLevel()
            end
            level_view:DeleteMe()
        end
        local mysize = math.ceil(loadSize / 1048576 * 10) / 10
        level_view:bindFunc(closeUpdateLevel, startloadLevel)
        level_view:setTitle(mysize)
    else
        UpdateEnd()
    end
end

--[[
    @brief: 资源更新的进度
]]
function GameUpdate:PatchUpdate(state, succeed, finished, percent, speed)
    --print("===========PatchUpdate: ", state, succeed, finished, percent, speed)
    --资源更新包
    if state == Config.Update.VERSION_DIFFTYPE.RESOURCE_DIFF then
        if not succeed then
                self.delay_call = {}
                self.delay_call.func = function ( )
                    VersionManager:getInstance():LoadNextDiffPatch(self.patch_i)
                end
                self.delay_call.beg_time = Status.NowTime
                self.delay_call.delay = 1.0
        else
            if not finished then
                local size = self.version_info.diff_patch_array[self.patch_i]/1024
                local pack_count = self.version_info.diff_pack_count
                local tips
                local p = percent
                if GameUpdate.load_level_num then
                    tips = string.format(Config.Update.WORDS[1], GameUpdate.load_level_num, size*percent, size, speed)
                else
                    local total_loaded_size = self.version_info.loaded_size + size * percent
                    tips = string.format(Config.Update.WORDS[2], total_loaded_size, self.version_info.total_size, speed)
                    p = total_loaded_size/self.version_info.total_size
                end
                self.update_text:setString(tips)
                self:setProgress(p * 100)
            else
                self:updateNotice()
            end
        end
    --安装更新包
    elseif state == Config.Update.VERSION_DIFFTYPE.PATCH_PACK then
        if not succeed then
            self.once_frame_action = function()
                 VersionManager:getInstance():LoadNextDiffPatch(self.patch_i)
             end
        else
            if GameUpdate.load_level_num then
                self.delay_call = {}
                self.delay_call.func = function()
                    self:UpdateFinished()
                end
                self.delay_call.beg_time = Status.NowTime
                self.delay_call.delay = 1.0
                return
            end
            if finished then
                if self.patch_i == self.version_info.diff_pack_count then
                    self.once_frame_action = function()
                        self:UpdateFinished()
                    end
                else
                    self.version_info.loaded_size = self.version_info.loaded_size + self.version_info.diff_patch_array[self.patch_i] / 1024
                    self.patch_i = self.patch_i + 1
                     VersionManager:getInstance():LoadNextDiffPatch(self.patch_i)
                end
            end

            local tips = string.format(Config.Update.WORDS[3])
            self.update_text:setString(tips)
            --self:setProgress(percent * 100)
        end
    --初始化游戏
    elseif state == Config.Update.VERSION_DIFFTYPE.INITIALIZATION then
        if not succeed then
            SAlert.New(Config.Update.LOGIN_ALERT[1], function() cc.Director:getInstance():endToLua() end, Config.Update.CONST_DES[4])
        else
            if not finished then
                local tips = Config.Update.LOGIN_ALERT[8]
                self.update_text:setString(tips)
                self:setProgress(percent * 100)
            else
                self.once_frame_action = function ()
                    self:retryVersionInfo()
                end
            end
        end
    elseif state == Config.Update.VERSION_DIFFTYPE.APP_DIFF then
        if not succeed then
                self:setProgress(0)
                self.delay_call = {}
                self.delay_call.func = function ()
                    self:loadNewApp()
                end
                self.delay_call.beg_time = Status.NowTime
                self.delay_call.delay = 1.0
        else
            if not finished then
                local  size = self.version_info.app_size/1024
                local tips = string.format(Config.Update.WORDS[4], 1, 1, percent*size,size,speed)
                self.update_text:setString(tips)
                self:setProgress(percent * 100)
            else
                self.once_frame_action = function ()
                    self:UpdateFinished()
                end
            end
        end
    end
end

--[[
    @brief: 加载新的更新包
]]
function GameUpdate:loadNewApp()
    -- local function startLoadApp()
    --     VersionManager:getInstance():LoadNewAppPatch(Config.PHP.PlatID)
    -- end
    -- local function skipLoadApp()
    --     self:UpdateFinished()
    -- end
    -- SAlert.New(Config.Update.LOGIN_ALERT[3], startLoadApp, Config.Update.CONST_DES[5])
    local function confirm()
        cc.Application:getInstance():jumpToURL("fantasy.yeahworld.com")
        cc.Director:getInstance():endToLua()
    end
    local function cancel()
        cc.Director:getInstance():endToLua()
    end
    SAlert.New(Config.Update.WORDS[5], confirm, Config.Update.WORDS[6], cancel, Config.Update.WORDS[7])
end

--[[
    @brief: 检查更新包
]]
function GameUpdate:checkUpdate()
    self.update_text:setString(Config.Update.LOGIN_ALERT[7])

    local local_version = self.version_info.local_version + 1000
    local remote_version = self.version_info.remote_version + 1000

    GameUpdate.local_app_version = self.version_info.local_app_version
    GameUpdate.local_version = self.version_info.local_version
    
    self.local_version:setString("v"..self.version_info.local_app_version..'.'..self.version_info.local_version)
    self.remote_version:setString("v"..self.version_info.remote_app_version..'.'..self.version_info.remote_version)

    local diff_result = self.version_info.result
    if diff_result == Config.Update.VERSION_DIFFTYPE.UP_TO_DATE then
         self.once_frame_action = function (  )
            self:UpdateFinished()
        end
    elseif diff_result == Config.Update.VERSION_DIFFTYPE.RESOURCE_DIFF then
        self.patch_i = 1
        self.once_frame_action = function (  )
            local tips = string.format(Config.Update.WORDS[3])
            self.update_text:setString(tips)
            self:setProgress(0)
            VersionManager:getInstance():LoadNextDiffPatch(self.patch_i)
        end
    elseif diff_result == Config.Update.VERSION_DIFFTYPE.APP_DIFF then
        self:loadNewApp()
    end

end

--[[
    @brief: 更新公告
]]
function GameUpdate:updateNotice()
    if self.isUpdateNotice then
        return
    end
    self.isUpdateNotice = true
    isgameupdate = true
    local time = os.time()
    cc.UserDefault:getInstance():setStringForKey("UPDATENOTICE_TIME",tostring(time))
    cc.UserDefault:getInstance():flush()
end